package javax.swing;

import java.awt.Graphics;
import java.awt.Shape;
import javax.swing.text.GlyphView;
import javax.swing.text.CompositeView;
import javax.swing.text.ParagraphView;
import javax.swing.text.BoxView;
import javax.swing.text.WrappedPlainView;
import javax.swing.text.View;
import javax.swing.text.ViewFactory;
import javax.swing.text.DefaultEditorKit;
import java.util.Vector;
import java.net.MalformedURLException;
import javax.accessibility.AccessibleHyperlink;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Element;
import javax.swing.text.AttributeSet;
import javax.swing.text.ElementIterator;
import javax.accessibility.AccessibleHypertext;
import javax.accessibility.AccessibleComponent;
import java.awt.IllegalComponentStateException;
import java.awt.Point;
import javax.accessibility.Accessible;
import javax.accessibility.AccessibleText;
import javax.accessibility.AccessibleState;
import javax.accessibility.AccessibleStateSet;
import java.util.HashMap;
import javax.accessibility.AccessibleContext;
import java.io.ObjectOutputStream;
import java.io.StringWriter;
import java.io.StringReader;
import javax.swing.plaf.TextUI;
import java.awt.Dimension;
import java.util.Iterator;
import javax.swing.text.Caret;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.html.HTML;
import java.io.DataOutputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.URLConnection;
import java.net.HttpURLConnection;
import javax.swing.text.ChangedCharSetException;
import javax.swing.text.BadLocationException;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.AbstractDocument;
import java.util.Enumeration;
import javax.swing.text.Document;
import java.io.InputStream;
import java.awt.Rectangle;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import java.io.IOException;
import java.awt.Component;
import java.awt.Container;
import java.util.Map;
import java.util.Hashtable;
import javax.swing.text.EditorKit;
import java.net.URL;
import javax.swing.text.JTextComponent;

public class JEditorPane extends JTextComponent
{
    private SwingWorker<URL, Object> pageLoader;
    private EditorKit kit;
    private boolean isUserSetEditorKit;
    private Hashtable<String, Object> pageProperties;
    static final String PostDataProperty = "javax.swing.JEditorPane.postdata";
    private Hashtable<String, EditorKit> typeHandlers;
    private static final Object kitRegistryKey;
    private static final Object kitTypeRegistryKey;
    private static final Object kitLoaderRegistryKey;
    private static final String uiClassID = "EditorPaneUI";
    public static final String W3C_LENGTH_UNITS = "JEditorPane.w3cLengthUnits";
    public static final String HONOR_DISPLAY_PROPERTIES = "JEditorPane.honorDisplayProperties";
    static final Map<String, String> defaultEditorKitMap;
    
    public JEditorPane() {
        this.setFocusCycleRoot(true);
        this.setFocusTraversalPolicy(new LayoutFocusTraversalPolicy() {
            @Override
            public Component getComponentAfter(Container container, Component component) {
                if (container != JEditorPane.this || (!JEditorPane.this.isEditable() && JEditorPane.this.getComponentCount() > 0)) {
                    return super.getComponentAfter(container, component);
                }
                Container focusCycleRootAncestor = JEditorPane.this.getFocusCycleRootAncestor();
                return (focusCycleRootAncestor != null) ? focusCycleRootAncestor.getFocusTraversalPolicy().getComponentAfter(focusCycleRootAncestor, JEditorPane.this) : null;
            }
            
            @Override
            public Component getComponentBefore(Container container, Component component) {
                if (container != JEditorPane.this || (!JEditorPane.this.isEditable() && JEditorPane.this.getComponentCount() > 0)) {
                    return super.getComponentBefore(container, component);
                }
                Container focusCycleRootAncestor = JEditorPane.this.getFocusCycleRootAncestor();
                return (focusCycleRootAncestor != null) ? focusCycleRootAncestor.getFocusTraversalPolicy().getComponentBefore(focusCycleRootAncestor, JEditorPane.this) : null;
            }
            
            @Override
            public Component getDefaultComponent(Container container) {
                return (container != JEditorPane.this || (!JEditorPane.this.isEditable() && JEditorPane.this.getComponentCount() > 0)) ? super.getDefaultComponent(container) : null;
            }
            
            @Override
            protected boolean accept(Component component) {
                return component != JEditorPane.this && super.accept(component);
            }
        });
        LookAndFeel.installProperty(this, "focusTraversalKeysForward", JComponent.getManagingFocusForwardTraversalKeys());
        LookAndFeel.installProperty(this, "focusTraversalKeysBackward", JComponent.getManagingFocusBackwardTraversalKeys());
    }
    
    public JEditorPane(URL page) throws IOException {
        this();
        this.setPage(page);
    }
    
    public JEditorPane(String page) throws IOException {
        this();
        this.setPage(page);
    }
    
    public JEditorPane(String contentType, String text) {
        this();
        this.setContentType(contentType);
        this.setText(text);
    }
    
    public synchronized void addHyperlinkListener(HyperlinkListener l) {
        this.listenerList.add(HyperlinkListener.class, l);
    }
    
    public synchronized void removeHyperlinkListener(HyperlinkListener l) {
        this.listenerList.remove(HyperlinkListener.class, l);
    }
    
    public synchronized HyperlinkListener[] getHyperlinkListeners() {
        return (HyperlinkListener[])this.listenerList.getListeners(HyperlinkListener.class);
    }
    
    public void fireHyperlinkUpdate(HyperlinkEvent hyperlinkEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == HyperlinkListener.class) {
                ((HyperlinkListener)listenerList[i + 1]).hyperlinkUpdate(hyperlinkEvent);
            }
        }
    }
    
    public void setPage(URL url) throws IOException {
        if (url == null) {
            throw new IOException("invalid url");
        }
        URL page = this.getPage();
        if (!url.equals(page) && url.getRef() == null) {
            this.scrollRectToVisible(new Rectangle(0, 0, 1, 1));
        }
        boolean n = false;
        Object postData = this.getPostData();
        if (page == null || !page.sameFile(url) || postData != null) {
            if (this.getAsynchronousLoadPriority(this.getDocument()) < 0) {
                InputStream stream = this.getStream(url);
                if (this.kit != null) {
                    Document initializeModel = this.initializeModel(this.kit, url);
                    if (this.getAsynchronousLoadPriority(initializeModel) >= 0) {
                        this.setDocument(initializeModel);
                        synchronized (this) {
                            (this.pageLoader = new JEditorPane.PageLoader(initializeModel, stream, page, url)).execute();
                        }
                        return;
                    }
                    this.read(stream, initializeModel);
                    this.setDocument(initializeModel);
                    n = true;
                }
            }
            else {
                if (this.pageLoader != null) {
                    this.pageLoader.cancel(true);
                }
                (this.pageLoader = new JEditorPane.PageLoader(null, null, page, url)).execute();
                return;
            }
        }
        final String ref = url.getRef();
        if (ref != null) {
            if (n == 0) {
                this.scrollToReference(ref);
            }
            else {
                SwingUtilities.invokeLater(new Runnable() {
                    @Override
                    public void run() {
                        JEditorPane.this.scrollToReference(ref);
                    }
                });
            }
            this.getDocument().putProperty("stream", url);
        }
        this.firePropertyChange("page", page, url);
        return;
    }
    
    private Document initializeModel(EditorKit editorKit, URL url) {
        Document defaultDocument = editorKit.createDefaultDocument();
        if (this.pageProperties != null) {
            Enumeration<String> keys = this.pageProperties.keys();
            while (keys.hasMoreElements()) {
                String key = (String)keys.nextElement();
                defaultDocument.putProperty(key, this.pageProperties.get(key));
            }
            this.pageProperties.clear();
        }
        if (defaultDocument.getProperty("stream") == null) {
            defaultDocument.putProperty("stream", url);
            return defaultDocument;
        }
        return defaultDocument;
    }
    
    private int getAsynchronousLoadPriority(Document document) {
        return (document instanceof AbstractDocument) ? ((AbstractDocument)document).getAsynchronousLoadPriority() : -1;
    }
    
    public void read(InputStream inputStream, Object desc) throws IOException {
        if (desc instanceof HTMLDocument && this.kit instanceof HTMLEditorKit) {
            HTMLDocument document = (HTMLDocument)desc;
            this.setDocument(document);
            this.read(inputStream, document);
        }
        else {
            String charsetName = (String)this.getClientProperty("charset");
            super.read((charsetName != null) ? new InputStreamReader(inputStream, charsetName) : new InputStreamReader(inputStream), desc);
        }
    }
    
    void read(InputStream inputStream, Document document) throws IOException {
        if (!Boolean.TRUE.equals(document.getProperty("IgnoreCharsetDirective"))) {
            inputStream = new BufferedInputStream(inputStream, 10240);
            inputStream.mark(10240);
        }
        try {
            String charsetName = (String)this.getClientProperty("charset");
            this.kit.read((charsetName != null) ? new InputStreamReader(inputStream, charsetName) : new InputStreamReader(inputStream), document, 0);
        }
        catch (BadLocationException ex) {
            throw new IOException(ex.getMessage());
        }
        catch (ChangedCharSetException ex2) {
            String charSetSpec = ex2.getCharSetSpec();
            if (ex2.keyEqualsCharSet()) {
                this.putClientProperty("charset", charSetSpec);
            }
            else {
                this.setCharsetFromContentTypeParameters(charSetSpec);
            }
            try {
                inputStream.reset();
            }
            catch (IOException ex3) {
                inputStream.close();
                URL url = (URL)document.getProperty("stream");
                if (url != null) {
                    inputStream = url.openConnection().getInputStream();
                }
                else {
                    throw ex2;
                }
            }
            try {
                document.remove(0, document.getLength());
            }
            catch (BadLocationException ex4) {}
            document.putProperty("IgnoreCharsetDirective", Boolean.valueOf(true));
            this.read(inputStream, document);
        }
    }
    
    protected InputStream getStream(URL context) throws IOException {
        final URLConnection openConnection = context.openConnection();
        if ((openConnection instanceof HttpURLConnection)) {
            HttpURLConnection httpURLConnection = (HttpURLConnection)openConnection;
            httpURLConnection.setInstanceFollowRedirects(false);
            Object postData = this.getPostData();
            if (postData != null) {
                this.handlePostData(httpURLConnection, postData);
            }
            int responseCode = httpURLConnection.getResponseCode();
            if (responseCode >= 300 && responseCode <= 399) {
                String headerField = openConnection.getHeaderField("Location");
                if (headerField.startsWith("http", 0)) {
                    context = new URL(headerField);
                }
                else {
                    context = new URL(context, headerField);
                }
                return this.getStream(context);
            }
        }
        if (SwingUtilities.isEventDispatchThread()) {
            this.handleConnectionProperties(openConnection);
        }
        else {
            try {
                SwingUtilities.invokeAndWait(new Runnable() {
                    @Override
                    public void run() {
                        JEditorPane.access$300(JEditorPane.this, openConnection);
                    }
                });
            }
            catch (InterruptedException cause) {
                throw new RuntimeException(cause);
            }
            catch (InvocationTargetException cause2) {
                throw new RuntimeException(cause2);
            }
        }
        return openConnection.getInputStream();
    }
    
    private void handleConnectionProperties(URLConnection urlConnection) {
        if (this.pageProperties == null) {
            this.pageProperties = new Hashtable();
        }
        String contentType = urlConnection.getContentType();
        if (contentType != null) {
            this.setContentType(contentType);
            this.pageProperties.put("content-type", contentType);
        }
        this.pageProperties.put("stream", urlConnection.getURL());
        String contentEncoding = urlConnection.getContentEncoding();
        if (contentEncoding != null) {
            this.pageProperties.put("content-encoding", contentEncoding);
        }
    }
    
    private Object getPostData() {
        return this.getDocument().getProperty("javax.swing.JEditorPane.postdata");
    }
    
    private void handlePostData(HttpURLConnection httpURLConnection, Object o) throws IOException {
        httpURLConnection.setDoOutput(true);
        DataOutputStream dataOutputStream = null;
        try {
            httpURLConnection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            dataOutputStream = new DataOutputStream(httpURLConnection.getOutputStream());
            dataOutputStream.writeBytes((String)o);
        }
        finally {
            if (dataOutputStream != null) {
                dataOutputStream.close();
            }
        }
    }
    
    public void scrollToReference(String anObject) {
        Document document = this.getDocument();
        if ((document instanceof HTMLDocument)) {
            HTMLDocument.Iterator iterator = ((HTMLDocument)document).getIterator(HTML.Tag.A);
            while (iterator.isValid()) {
                String s = (String)iterator.getAttributes().getAttribute(HTML.Attribute.NAME);
                if (s != null && s.equals(anObject)) {
                    try {
                        int startOffset = iterator.getStartOffset();
                        Rectangle modelToView = this.modelToView(startOffset);
                        if (modelToView != null) {
                            modelToView.height = this.getVisibleRect().height;
                            this.scrollRectToVisible(modelToView);
                            this.setCaretPosition(startOffset);
                        }
                    }
                    catch (BadLocationException ex) {
                        UIManager.getLookAndFeel().provideErrorFeedback(this);
                    }
                }
                iterator.next();
            }
        }
    }
    
    public URL getPage() {
        return (URL)this.getDocument().getProperty("stream");
    }
    
    public void setPage(String spec) throws IOException {
        if (spec == null) {
            throw new IOException("invalid url");
        }
        this.setPage(new URL(spec));
        return;
    }
    
    @Override
    public String getUIClassID() {
        return "EditorPaneUI";
    }
    
    protected EditorKit createDefaultEditorKit() {
        return new JEditorPane.PlainEditorKit();
    }
    
    public EditorKit getEditorKit() {
        if (this.kit == null) {
            this.kit = this.createDefaultEditorKit();
            this.isUserSetEditorKit = false;
        }
        return this.kit;
    }
    
    public final String getContentType() {
        return (this.kit != null) ? this.kit.getContentType() : null;
    }
    
    public final void setContentType(String trim) {
        int index = trim.indexOf(";");
        if (index > -1) {
            String substring = trim.substring(index);
            trim = trim.substring(0, index).trim();
            if (trim.toLowerCase().startsWith("text/")) {
                this.setCharsetFromContentTypeParameters(substring);
            }
        }
        if (this.kit == null || !trim.equals(this.kit.getContentType()) || !this.isUserSetEditorKit) {
            EditorKit editorKitForContentType = this.getEditorKitForContentType(trim);
            if (editorKitForContentType != null && editorKitForContentType != this.kit) {
                this.setEditorKit(editorKitForContentType);
                this.isUserSetEditorKit = false;
            }
        }
    }
    
    private void setCharsetFromContentTypeParameters(String substring) {
        try {
            int index = substring.indexOf(59);
            if (index > -1 && index < substring.length() - 1) {
                substring = substring.substring(index + 1);
            }
            if (substring.length() > 0) {
                String value = new HeaderParser(substring).findValue("charset");
                if (value != null) {
                    this.putClientProperty("charset", value);
                }
            }
        }
        catch (IndexOutOfBoundsException ex) {}
        catch (NullPointerException ex2) {}
        catch (Exception ex3) {
            System.err.println("JEditorPane.getCharsetFromContentTypeParameters failed on: " + substring);
            ex3.printStackTrace();
        }
    }
    
    public void setEditorKit(EditorKit editorKit) {
        EditorKit kit = this.kit;
        this.isUserSetEditorKit = true;
        if (kit != null) {
            kit.deinstall(this);
        }
        this.kit = editorKit;
        if (this.kit != null) {
            this.kit.install(this);
            this.setDocument(this.kit.createDefaultDocument());
        }
        this.firePropertyChange("editorKit", kit, editorKit);
    }
    
    public EditorKit getEditorKitForContentType(String key) {
        if (this.typeHandlers == null) {
            this.typeHandlers = new Hashtable(3);
        }
        EditorKit editorKit = (EditorKit)this.typeHandlers.get(key);
        if (editorKit == null) {
            editorKit = createEditorKitForContentType(key);
            if (editorKit != null) {
                this.setEditorKitForContentType(key, editorKit);
            }
        }
        if (editorKit == null) {
            editorKit = this.createDefaultEditorKit();
            return editorKit;
        }
        return editorKit;
    }
    
    public void setEditorKitForContentType(String key, EditorKit value) {
        if (this.typeHandlers == null) {
            this.typeHandlers = new Hashtable(3);
        }
        this.typeHandlers.put(key, value);
    }
    
    @Override
    public void replaceSelection(String s) {
        if (!this.isEditable()) {
            UIManager.getLookAndFeel().provideErrorFeedback(this);
            return;
        }
        EditorKit editorKit = this.getEditorKit();
        if ((editorKit instanceof StyledEditorKit)) {
            try {
                Document document = this.getDocument();
                Caret caret = this.getCaret();
                int saveComposedText = this.saveComposedText(caret.getDot());
                int min = Math.min(caret.getDot(), caret.getMark());
                int max = Math.max(caret.getDot(), caret.getMark());
                if ((document instanceof AbstractDocument)) {
                    ((AbstractDocument)document).replace(min, max - min, s, ((StyledEditorKit)editorKit).getInputAttributes());
                }
                else {
                    if (min != max) {
                        document.remove(min, max - min);
                    }
                    if (s != null && s.length() > 0) {
                        document.insertString(min, s, ((StyledEditorKit)editorKit).getInputAttributes());
                    }
                }
                if (!(saveComposedText == 0)) {
                    this.restoreComposedText();
                }
            }
            catch (BadLocationException ex) {
                UIManager.getLookAndFeel().provideErrorFeedback(this);
            }
        }
        else {
            super.replaceSelection(s);
        }
        return;
    }
    
    public static EditorKit createEditorKitForContentType(String s) {
        Hashtable<String, EditorKit> kitRegisty = getKitRegisty();
        EditorKit value = (EditorKit)kitRegisty.get(s);
        if (value == null) {
            String s2 = (String)getKitTypeRegistry().get(s);
            ClassLoader classLoader = (ClassLoader)getKitLoaderRegistry().get(s);
            try {
                Class<Object> clazz;
                if (classLoader != null) {
                    clazz = classLoader.loadClass(s2);
                }
                else {
                    clazz = Class.forName(s2, true, Thread.currentThread().getContextClassLoader());
                }
                value = (EditorKit)clazz.newInstance();
                kitRegisty.put(s, value);
            }
            catch (Throwable t) {
                value = null;
            }
        }
        if (value != null) {
            return (EditorKit)value.clone();
        }
        return null;
    }
    
    public static void registerEditorKitForContentType(String s, String s2) {
        registerEditorKitForContentType(s, s2, Thread.currentThread().getContextClassLoader());
    }
    
    public static void registerEditorKitForContentType(String s, String value, ClassLoader value2) {
        getKitTypeRegistry().put(s, value);
        if (value2 != null) {
            getKitLoaderRegistry().put(s, value2);
        }
        else {
            getKitLoaderRegistry().remove(s);
        }
        getKitRegisty().remove(s);
    }
    
    public static String getEditorKitClassNameForContentType(String key) {
        return (String)getKitTypeRegistry().get(key);
    }
    
    private static Hashtable<String, String> getKitTypeRegistry() {
        loadDefaultKitsIfNecessary();
        return (Hashtable)SwingUtilities.appContextGet(JEditorPane.kitTypeRegistryKey);
    }
    
    private static Hashtable<String, ClassLoader> getKitLoaderRegistry() {
        loadDefaultKitsIfNecessary();
        return (Hashtable)SwingUtilities.appContextGet(JEditorPane.kitLoaderRegistryKey);
    }
    
    private static Hashtable<String, EditorKit> getKitRegisty() {
        Hashtable hashtable = (Hashtable)SwingUtilities.appContextGet(JEditorPane.kitRegistryKey);
        if (hashtable == null) {
            hashtable = new Hashtable(3);
            SwingUtilities.appContextPut(JEditorPane.kitRegistryKey, hashtable);
            return hashtable;
        }
        return hashtable;
    }
    
    private static void loadDefaultKitsIfNecessary() {
        if (SwingUtilities.appContextGet(JEditorPane.kitTypeRegistryKey) == null) {
            synchronized (JEditorPane.defaultEditorKitMap) {
                if (JEditorPane.defaultEditorKitMap.size() == 0) {
                    JEditorPane.defaultEditorKitMap.put("text/plain", "javax.swing.JEditorPane$PlainEditorKit");
                    JEditorPane.defaultEditorKitMap.put("text/html", "javax.swing.text.html.HTMLEditorKit");
                    JEditorPane.defaultEditorKitMap.put("text/rtf", "javax.swing.text.rtf.RTFEditorKit");
                    JEditorPane.defaultEditorKitMap.put("application/rtf", "javax.swing.text.rtf.RTFEditorKit");
                }
            }
            SwingUtilities.appContextPut(JEditorPane.kitTypeRegistryKey, new Hashtable());
            SwingUtilities.appContextPut(JEditorPane.kitLoaderRegistryKey, new Hashtable());
            for (String s : JEditorPane.defaultEditorKitMap.keySet()) {
                registerEditorKitForContentType(s, (String)JEditorPane.defaultEditorKitMap.get(s));
            }
        }
    }
    
    @Override
    public Dimension getPreferredSize() {
        Dimension preferredSize = super.getPreferredSize();
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        if ((unwrappedParent instanceof JViewport)) {
            JViewport viewport = (JViewport)unwrappedParent;
            TextUI ui = this.getUI();
            int width = preferredSize.width;
            int height = preferredSize.height;
            if (!this.getScrollableTracksViewportWidth()) {
                int width2 = viewport.getWidth();
                Dimension minimumSize = ui.getMinimumSize(this);
                if (width2 != 0 && width2 < minimumSize.width) {
                    width = minimumSize.width;
                }
            }
            if (!this.getScrollableTracksViewportHeight()) {
                int height2 = viewport.getHeight();
                Dimension minimumSize2 = ui.getMinimumSize(this);
                if (height2 != 0 && height2 < minimumSize2.height) {
                    height = minimumSize2.height;
                }
            }
            if (width != preferredSize.width || height != preferredSize.height) {
                preferredSize = new Dimension(width, height);
            }
        }
        return preferredSize;
    }
    
    @Override
    public void setText(String s) {
        try {
            Document document = this.getDocument();
            document.remove(0, document.getLength());
            if (s == null || s.equals("")) {
                return;
            }
            this.getEditorKit().read(new StringReader(s), document, 0);
        }
        catch (IOException ex) {
            UIManager.getLookAndFeel().provideErrorFeedback(this);
        }
        catch (BadLocationException ex2) {
            UIManager.getLookAndFeel().provideErrorFeedback(this);
        }
    }
    
    @Override
    public String getText() {
        String string;
        try {
            StringWriter out = new StringWriter();
            this.write(out);
            string = out.toString();
        }
        catch (IOException ex) {
            string = null;
        }
        return string;
    }
    
    @Override
    public boolean getScrollableTracksViewportWidth() {
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        if ((unwrappedParent instanceof JViewport)) {
            JViewport viewport = (JViewport)unwrappedParent;
            TextUI ui = this.getUI();
            int width = viewport.getWidth();
            Dimension minimumSize = ui.getMinimumSize(this);
            Dimension maximumSize = ui.getMaximumSize(this);
            if (width >= minimumSize.width && width <= maximumSize.width) {
                return true;
            }
        }
        return false;
    }
    
    @Override
    public boolean getScrollableTracksViewportHeight() {
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        if ((unwrappedParent instanceof JViewport)) {
            JViewport viewport = (JViewport)unwrappedParent;
            TextUI ui = this.getUI();
            int height = viewport.getHeight();
            if (height >= ui.getMinimumSize(this).height && height <= ui.getMaximumSize(this).height) {
                return true;
            }
        }
        return false;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("EditorPaneUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",kit=" + ((this.kit != null) ? this.kit.toString() : "") + ",typeHandlers=" + ((this.typeHandlers != null) ? this.typeHandlers.toString() : "");
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if ((this.getEditorKit() instanceof HTMLEditorKit)) {
            if (this.accessibleContext == null || this.accessibleContext.getClass() != JEditorPane.AccessibleJEditorPaneHTML.class) {
                this.accessibleContext = new JEditorPane.AccessibleJEditorPaneHTML();
            }
        }
        else if (this.accessibleContext == null || this.accessibleContext.getClass() != JEditorPane.AccessibleJEditorPane.class) {
            this.accessibleContext = new JEditorPane.AccessibleJEditorPane();
        }
        return this.accessibleContext;
    }
    
    static Document access$100(JEditorPane editorPane, EditorKit editorKit, URL url) {
        return editorPane.initializeModel(editorKit, url);
    }
    
    static void access$200(JEditorPane editorPane, String propertyName, Object oldValue, Object newValue) {
        editorPane.firePropertyChange(propertyName, oldValue, newValue);
    }
    
    static void access$300(JEditorPane editorPane, URLConnection urlConnection) {
        editorPane.handleConnectionProperties(urlConnection);
    }
    
    static {
        kitRegistryKey = new StringBuffer("JEditorPane.kitRegistry");
        kitTypeRegistryKey = new StringBuffer("JEditorPane.kitTypeRegistry");
        kitLoaderRegistryKey = new StringBuffer("JEditorPane.kitLoaderRegistry");
        defaultEditorKitMap = new HashMap(0);
    }
    
    class PageLoader extends SwingWorker<URL, Object>
    {
        InputStream in;
        URL old;
        URL page;
        Document doc;
        
        PageLoader(Document doc, InputStream in, URL old, URL page) {
            this.in = in;
            this.old = old;
            this.page = page;
            this.doc = doc;
        }
        
        @Override
        protected URL doInBackground() {
            int n = 0;
            try {
                if (this.in == null) {
                    this.in = JEditorPane.this.getStream(this.page);
                    if (JEditorPane.this.kit == null) {
                        UIManager.getLookAndFeel().provideErrorFeedback(JEditorPane.this);
                        URL old = this.old;
                        if (n != 0) {
                            SwingUtilities.invokeLater(new Runnable() {
                                @Override
                                public void run() {
                                    JEditorPane.access$200(JEditorPane.this, "page", JEditorPane.PageLoader.this.old, JEditorPane.PageLoader.this.page);
                                }
                            });
                        }
                        return (n != 0) ? this.page : old;
                    }
                }
                if (this.doc == null) {
                    try {
                        SwingUtilities.invokeAndWait(new Runnable() {
                            @Override
                            public void run() {
                                JEditorPane.PageLoader.this.doc = JEditorPane.access$100(JEditorPane.this, JEditorPane.this.kit, JEditorPane.PageLoader.this.page);
                                JEditorPane.this.setDocument(JEditorPane.PageLoader.this.doc);
                            }
                        });
                    }
                    catch (InvocationTargetException ex) {
                        UIManager.getLookAndFeel().provideErrorFeedback(JEditorPane.this);
                        URL old2 = this.old;
                        if (n != 0) {
                            SwingUtilities.invokeLater(new Runnable() {
                                @Override
                                public void run() {
                                    JEditorPane.access$200(JEditorPane.this, "page", JEditorPane.PageLoader.this.old, JEditorPane.PageLoader.this.page);
                                }
                            });
                        }
                        return (n != 0) ? this.page : old2;
                    }
                    catch (InterruptedException ex2) {
                        UIManager.getLookAndFeel().provideErrorFeedback(JEditorPane.this);
                        URL old3 = this.old;
                        if (n != 0) {
                            SwingUtilities.invokeLater(new Runnable() {
                                @Override
                                public void run() {
                                    JEditorPane.access$200(JEditorPane.this, "page", JEditorPane.PageLoader.this.old, JEditorPane.PageLoader.this.page);
                                }
                            });
                        }
                        return (n != 0) ? this.page : old3;
                    }
                }
                JEditorPane.this.read(this.in, this.doc);
                if (((URL)this.doc.getProperty("stream")).getRef() != null) {
                    SwingUtilities.invokeLater(new Runnable() {
                        @Override
                        public void run() {
                            JEditorPane.this.scrollToReference(((URL)JEditorPane.this.getDocument().getProperty("stream")).getRef());
                        }
                    });
                }
                n = 1;
                if (n != 0) {
                    SwingUtilities.invokeLater(new Runnable() {
                        @Override
                        public void run() {
                            JEditorPane.access$200(JEditorPane.this, "page", JEditorPane.PageLoader.this.old, JEditorPane.PageLoader.this.page);
                        }
                    });
                }
                return (n != 0) ? this.page : n;
            }
            catch (IOException ex3) {
                UIManager.getLookAndFeel().provideErrorFeedback(JEditorPane.this);
                if (n != 0) {
                    SwingUtilities.invokeLater(new Runnable() {
                        @Override
                        public void run() {
                            JEditorPane.access$200(JEditorPane.this, "page", JEditorPane.PageLoader.this.old, JEditorPane.PageLoader.this.page);
                        }
                    });
                }
                return (n != 0) ? this.page : this.old;
            }
            finally {
                if (n != 0) {
                    SwingUtilities.invokeLater(new Runnable() {
                        @Override
                        public void run() {
                            JEditorPane.access$200(JEditorPane.this, "page", JEditorPane.PageLoader.this.old, JEditorPane.PageLoader.this.page);
                        }
                    });
                }
                return (n != 0) ? this.page : this.old;
            }
        }
    }
    
    protected class AccessibleJEditorPane extends JTextComponent.AccessibleJTextComponent
    {
        @Override
        public String getAccessibleDescription() {
            String s = this.accessibleDescription;
            if (s == null) {
                s = (String)JEditorPane.this.getClientProperty("AccessibleDescription");
            }
            if (s == null) {
                s = JEditorPane.this.getContentType();
                return s;
            }
            return s;
        }
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
            accessibleStateSet.add(AccessibleState.MULTI_LINE);
            return accessibleStateSet;
        }
    }
    
    protected class AccessibleJEditorPaneHTML extends JEditorPane.AccessibleJEditorPane
    {
        private AccessibleContext accessibleContext;
        
        @Override
        public AccessibleText getAccessibleText() {
            return new JEditorPane.JEditorPaneAccessibleHypertextSupport();
        }
        
        protected AccessibleJEditorPaneHTML() {
            this.accessibleContext = ((HTMLEditorKit)JEditorPane.this.getEditorKit()).getAccessibleContext();
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            if (this.accessibleContext != null) {
                return this.accessibleContext.getAccessibleChildrenCount();
            }
            return 0;
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            if (this.accessibleContext != null) {
                return this.accessibleContext.getAccessibleChild(n);
            }
            return null;
        }
        
        @Override
        public Accessible getAccessibleAt(Point point) {
            if (this.accessibleContext != null && point != null) {
                try {
                    AccessibleComponent accessibleComponent = this.accessibleContext.getAccessibleComponent();
                    if (accessibleComponent != null) {
                        return accessibleComponent.getAccessibleAt(point);
                    }
                    return null;
                }
                catch (IllegalComponentStateException ex) {
                    return null;
                }
            }
            else {
                return null;
            }
        }
    }
    
    protected class JEditorPaneAccessibleHypertextSupport extends JEditorPane.AccessibleJEditorPane implements AccessibleHypertext
    {
        JEditorPane.JEditorPaneAccessibleHypertextSupport.LinkVector hyperlinks;
        boolean linksValid;
        
        private void buildLinkTable() {
            this.hyperlinks.removeAllElements();
            Document document = JEditorPane.this.getDocument();
            if (document != null) {
                Element next;
                while ((next = new ElementIterator(document).next()) != null) {
                    if (!next.isLeaf()) {
                        continue;
                    }
                    AttributeSet set = (AttributeSet)next.getAttributes().getAttribute(HTML.Tag.A);
                    if (((set != null) ? ((String)set.getAttribute(HTML.Attribute.HREF)) : null) == null) {
                        continue;
                    }
                    this.hyperlinks.addElement(new JEditorPane.JEditorPaneAccessibleHypertextSupport.HTMLLink(next));
                }
            }
            this.linksValid = true;
        }
        
        public JEditorPaneAccessibleHypertextSupport() {
            this.linksValid = false;
            this.hyperlinks = new JEditorPane.JEditorPaneAccessibleHypertextSupport.LinkVector();
            Document document = JEditorPane.this.getDocument();
            if (document != null) {
                document.addDocumentListener(new DocumentListener() {
                    @Override
                    public void changedUpdate(DocumentEvent documentEvent) {
                        JEditorPane.JEditorPaneAccessibleHypertextSupport.this.linksValid = false;
                    }
                    
                    @Override
                    public void insertUpdate(DocumentEvent documentEvent) {
                        JEditorPane.JEditorPaneAccessibleHypertextSupport.this.linksValid = false;
                    }
                    
                    @Override
                    public void removeUpdate(DocumentEvent documentEvent) {
                        JEditorPane.JEditorPaneAccessibleHypertextSupport.this.linksValid = false;
                    }
                });
            }
        }
        
        @Override
        public int getLinkCount() {
            if (!this.linksValid) {
                this.buildLinkTable();
            }
            return this.hyperlinks.size();
        }
        
        @Override
        public int getLinkIndex(int n) {
            if (!this.linksValid) {
                this.buildLinkTable();
            }
            Element element = null;
            Document document = JEditorPane.this.getDocument();
            if (document != null) {
                for (element = document.getDefaultRootElement(); !element.isLeaf(); element = element.getElement(element.getElementIndex(n))) {}
            }
            return this.hyperlinks.baseElementIndex(element);
        }
        
        @Override
        public AccessibleHyperlink getLink(int index) {
            if (!this.linksValid) {
                this.buildLinkTable();
            }
            if (index >= 0 && index < this.hyperlinks.size()) {
                return (AccessibleHyperlink)this.hyperlinks.elementAt(index);
            }
            return null;
        }
        
        public String getLinkText(int index) {
            if (!this.linksValid) {
                this.buildLinkTable();
            }
            Element element = (Element)this.hyperlinks.elementAt(index);
            if (element != null) {
                Document document = JEditorPane.this.getDocument();
                if (document != null) {
                    try {
                        return document.getText(element.getStartOffset(), element.getEndOffset() - element.getStartOffset());
                    }
                    catch (BadLocationException ex) {
                        return null;
                    }
                }
            }
            return null;
        }
        
        public class HTMLLink extends AccessibleHyperlink
        {
            Element element;
            
            public HTMLLink(Element element) {
                this.element = element;
            }
            
            @Override
            public boolean isValid() {
                return JEditorPane.JEditorPaneAccessibleHypertextSupport.this.linksValid;
            }
            
            @Override
            public int getAccessibleActionCount() {
                return 1;
            }
            
            @Override
            public boolean doAccessibleAction(int n) {
                if (n == 0 && this.isValid()) {
                    URL u = (URL)this.getAccessibleActionObject(n);
                    if (u != null) {
                        JEditorPane.this.fireHyperlinkUpdate(new HyperlinkEvent(JEditorPane.this, HyperlinkEvent.EventType.ACTIVATED, u));
                        return true;
                    }
                }
                return false;
            }
            
            @Override
            public String getAccessibleActionDescription(int n) {
                if (n == 0 && this.isValid()) {
                    Document document = JEditorPane.this.getDocument();
                    if (document != null) {
                        try {
                            return document.getText(this.getStartIndex(), this.getEndIndex() - this.getStartIndex());
                        }
                        catch (BadLocationException ex) {
                            return null;
                        }
                    }
                }
                return null;
            }
            
            @Override
            public Object getAccessibleActionObject(int n) {
                if (n == 0 && this.isValid()) {
                    AttributeSet set = (AttributeSet)this.element.getAttributes().getAttribute(HTML.Tag.A);
                    String spec = (set != null) ? ((String)set.getAttribute(HTML.Attribute.HREF)) : null;
                    if (spec != null) {
                        Object o;
                        try {
                            o = new URL(JEditorPane.this.getPage(), spec);
                        }
                        catch (MalformedURLException ex) {
                            o = null;
                        }
                        return o;
                    }
                }
                return null;
            }
            
            @Override
            public Object getAccessibleActionAnchor(int n) {
                return this.getAccessibleActionDescription(n);
            }
            
            @Override
            public int getStartIndex() {
                return this.element.getStartOffset();
            }
            
            @Override
            public int getEndIndex() {
                return this.element.getEndOffset();
            }
        }
        
        private class LinkVector extends Vector<JEditorPane.JEditorPaneAccessibleHypertextSupport.HTMLLink>
        {
            public int baseElementIndex(Element element) {
                int i = 0;
                while (i < this.elementCount) {
                    if (((JEditorPane.JEditorPaneAccessibleHypertextSupport.HTMLLink)this.elementAt(i)).element == element) {
                        return i;
                    }
                    ++i;
                }
                return -1;
            }
        }
    }
    
    static class PlainEditorKit extends DefaultEditorKit implements ViewFactory
    {
        @Override
        public ViewFactory getViewFactory() {
            return this;
        }
        
        @Override
        public View create(Element elem) {
            Object property = elem.getDocument().getProperty("i18n");
            if (property != null && property.equals(Boolean.TRUE)) {
                return this.createI18N(elem);
            }
            return new WrappedPlainView(elem);
        }
        
        View createI18N(Element elem) {
            String name = elem.getName();
            if (name != null) {
                if (name.equals("content")) {
                    return new PlainParagraph(elem);
                }
                if (name.equals("paragraph")) {
                    return new BoxView(elem, 1);
                }
            }
            return null;
        }
        
        static class PlainParagraph extends ParagraphView
        {
            PlainParagraph(Element elem) {
                super(elem);
                (this.layoutPool = new LogicalView(elem)).setParent(this);
            }
            
            @Override
            protected void setPropertiesFromAttributes() {
                Container container = this.getContainer();
                if (container != null && !container.getComponentOrientation().isLeftToRight()) {
                    this.setJustification(2);
                }
                else {
                    this.setJustification(0);
                }
            }
            
            @Override
            public int getFlowSpan(int index) {
                Container container = this.getContainer();
                if (container instanceof JTextArea && !((JTextArea)container).getLineWrap()) {
                    return Integer.MAX_VALUE;
                }
                return super.getFlowSpan(index);
            }
            
            @Override
            protected SizeRequirements calculateMinorAxisRequirements(int axis, SizeRequirements r) {
                SizeRequirements calculateMinorAxisRequirements = super.calculateMinorAxisRequirements(axis, r);
                Container container = this.getContainer();
                if (container instanceof JTextArea && !((JTextArea)container).getLineWrap()) {
                    calculateMinorAxisRequirements.minimum = calculateMinorAxisRequirements.preferred;
                    return calculateMinorAxisRequirements;
                }
                return calculateMinorAxisRequirements;
            }
            
            static class LogicalView extends CompositeView
            {
                LogicalView(Element elem) {
                    super(elem);
                }
                
                @Override
                protected int getViewIndexAtPosition(int n) {
                    Element element = this.getElement();
                    if (element.getElementCount() > 0) {
                        return element.getElementIndex(n);
                    }
                    return 0;
                }
                
                @Override
                protected boolean updateChildren(DocumentEvent.ElementChange elementChange, DocumentEvent documentEvent, ViewFactory viewFactory) {
                    return false;
                }
                
                @Override
                protected void loadChildren(ViewFactory f) {
                    Element element = this.getElement();
                    if (element.getElementCount() > 0) {
                        super.loadChildren(f);
                    }
                    else {
                        this.append(new GlyphView(element));
                    }
                }
                
                @Override
                public float getPreferredSpan(int n) {
                    if (this.getViewCount() != 1) {
                        throw new Error("One child view is assumed.");
                    }
                    return this.getView(0).getPreferredSpan(n);
                }
                
                @Override
                protected void forwardUpdateToView(View v, DocumentEvent e, Shape a, ViewFactory f) {
                    v.setParent(this);
                    super.forwardUpdateToView(v, e, a, f);
                }
                
                @Override
                public void paint(Graphics graphics, Shape shape) {
                }
                
                @Override
                protected boolean isBefore(int n, int n2, Rectangle rectangle) {
                    return false;
                }
                
                @Override
                protected boolean isAfter(int n, int n2, Rectangle rectangle) {
                    return false;
                }
                
                @Override
                protected View getViewAtPoint(int n, int n2, Rectangle rectangle) {
                    return null;
                }
                
                @Override
                protected void childAllocation(int n, Rectangle rectangle) {
                }
            }
        }
    }
    
    static class HeaderParser
    {
        String raw;
        String[][] tab;
        
        public HeaderParser(String raw) {
            this.raw = raw;
            this.tab = new String[10][2];
            this.parse();
        }
        
        private void parse() {
            if (this.raw != null) {
                this.raw = this.raw.trim();
                char[] charArray = this.raw.toCharArray();
                int offset = 0;
                int i = 0;
                int n = 0;
                int n2 = 1;
                int n3 = 0;
                int length = charArray.length;
                while (i < length) {
                    int n4 = charArray[i];
                    if (n4 == '=') {
                        this.tab[n][0] = new String(charArray, offset, i - offset).toLowerCase();
                        n2 = 0;
                        offset = ++i;
                    }
                    else if (n4 == '\"') {
                        if (n3 != 0) {
                            this.tab[n++][1] = new String(charArray, offset, i - offset);
                            n3 = 0;
                            while (++i < length && (charArray[i] == ' ' || charArray[i] == ',')) {}
                            n2 = 1;
                            offset = i;
                        }
                        else {
                            n3 = 1;
                            offset = ++i;
                        }
                    }
                    else if (n4 == ' ' || n4 == ',') {
                        if (n3 != 0) {
                            ++i;
                        }
                        else {
                            if (n2 != 0) {
                                this.tab[n++][0] = new String(charArray, offset, i - offset).toLowerCase();
                            }
                            else {
                                this.tab[n++][1] = new String(charArray, offset, i - offset);
                            }
                            while (i < length && (charArray[i] == ' ' || charArray[i] == ',')) {
                                ++i;
                            }
                            n2 = 1;
                            offset = i;
                        }
                    }
                    else {
                        ++i;
                    }
                }
                if (--i > offset) {
                    if (n2 == 0) {
                        if (charArray[i] == '\"') {
                            this.tab[n++][1] = new String(charArray, offset, i - offset);
                        }
                        else {
                            this.tab[n++][1] = new String(charArray, offset, i - offset + 1);
                        }
                    }
                    else {
                        this.tab[n][0] = new String(charArray, offset, i - offset + 1).toLowerCase();
                    }
                }
                else if (i == offset) {
                    if (n2 == 0) {
                        if (charArray[i] == '\"') {
                            this.tab[n++][1] = String.valueOf(charArray[i - 1]);
                        }
                        else {
                            this.tab[n++][1] = String.valueOf(charArray[i]);
                        }
                    }
                    else {
                        this.tab[n][0] = String.valueOf(charArray[i]).toLowerCase();
                    }
                }
            }
        }
        
        public String findKey(int n) {
            if (n < 0 || n > 10) {
                return null;
            }
            return this.tab[n][0];
        }
        
        public String findValue(int n) {
            if (n < 0 || n > 10) {
                return null;
            }
            return this.tab[n][1];
        }
        
        public String findValue(String s) {
            return this.findValue(s, null);
        }
        
        public String findValue(String lowerCase, String s) {
            if (lowerCase == null) {
                return s;
            }
            lowerCase = lowerCase.toLowerCase();
            int i = 0;
            while (i < 10) {
                if (this.tab[i][0] == null) {
                    return s;
                }
                if (lowerCase.equals(this.tab[i][0])) {
                    return this.tab[i][1];
                }
                ++i;
            }
            return s;
        }
        
        public int findInt(String s, int i) {
            try {
                return Integer.parseInt(this.findValue(s, String.valueOf(i)));
            }
            catch (Throwable t) {
                return i;
            }
        }
    }
}
